'********************************************
'*  NTSC 256 x 192 - 8 bit paletted buffer  *
'********************************************
' Based on NTSC Texture, adapted by lachlan@tubularcontrols.com
' Renders as 256x192x8bpp 32bit paletted. Uses video mode F (2:8:8:8)
' Adapted by Coley for general display buffer, removed unneccesary code for clarity
' Altered by Baggers to draw a paletted buffer for 256 colours of full 32bit goodness
' Added Fire :D

CON
  _clkmode = xinput
  _xinfreq = 60_000_000

  s		= 46		'scales DAC output (s = 0..128)  (adjust slightly to optimize slight color artifacts)
  r		= s * 78 / 128	'adjusts for modulator expansion
  lntsc         = 3640          'NTSC colorburst cycles per line * 16
  sntsc         = 624           'NTSC colorburst cycles per sync * 16
  linestot	= 96		'DE0 repeat every line twice to fit buffer in 24kB

DAT		org
		setctra	#%00111			'set ctra for PLL*8
                setfrqa	frqa_			'set frqa
		setvid	#%1110			'set vid for composite output
		setvidy	vidy_			'set yiq coefficients
		setvidi	vidi_
		setvidq	vidq_
		cfgdac0	#%11			'set bottom dac to video mode (YIQ)
		cfgpins	pinmask,pindacs		'set P0 for dac output

		setcog	#8
		coginit	fire_ptr,fire_par

'
' Field loop
'
field           mov     x,#34                   'top blanks
                call    #blank

		setptra	palptr
		reps	#256,#1
		setinda	#palette
		rdlong	inda++,ptra++

                setptra hubbuf                  'reset to start of screen buffer
                mov     y,#192                  'set visible lines
                
line            call    #hsync                  'do horizontal sync
                waitvid v_bo,#0                 'black left border

                waitvid mode256,#$00              '64*9 clocks per waitvid
                setspa  #$00
                call    #render256                 'do waitvid and render 1st 64 pixels

                waitvid v_bo,#0                 'black right border
                djnz    y,#line                 'another line?

		wrlong	one,VSyncPtr

                mov     x,#28                   'bottom blanks
                call    #blank

                mov     x,#6                    'high vertical syncs
                call    #vsynch
                mov     x,#6                    'low vertical syncs
                call    #vsyncl
                mov     x,#6                    'high vertical syncs
                call    #vsynch

                jmp     #field                  'field loop

blank           call    #hsync                  'blank lines
                waitvid v_bl,#0
                djnz    x,#blank
blank_ret       ret

hsync           waitvid v_bs,c_sh               'horizontal sync
                waitvid v_sn,c_sl
hsync_ret       retd
                waitvid v_bc,c_sh
                waitvid v_cb,c_cb
                waitvid v_bv,c_sh

vsynch          waitvid v_bs,c_sh               'vertical sync high
                waitvid v_hs,c_sl
                waitvid v_hr,c_sh
                djnz    x,#vsynch
vsynch_ret      ret

vsyncl          waitvid v_bs,c_sh               'vertical sync low
                waitvid v_ls,c_sl
                waitvid v_lr,c_sh
                djnz    x,#vsyncl
vsyncl_ret      ret

'
' Render pixels
'
render256       mov     tmp3,#256		'loop counter, 256 pixels
:lp             rdbytec tmp2, ptra++		'remember there is only one PTRA/PTRB (for multi threading)
		add	tmp2,#palette		'add palette offset in cogram
		movs	:mod,tmp2		'set source of :mod
		djnzd	tmp3,#:lp		'do loop ( delayed )
		nop				'nop timing
:mod		mov	tmp,0-0			'get palette into tmp
	        pusha   tmp			'store in stack ram (2:8:8:8)
render256_ret   ret

'
' Initialized data - video
'

frqa_		long	$1E8B_A2A7		'57.272720 MHz (colorburst * 16)
vidy_		long	(110*s/128 & $FF) << 24 + ((+38*s/128) & $FF) << 16 + ((+75*s/128) & $FF) << 8 + ((+15*s/128) & $FF)
vidi_		long	(100*s/128 & $FF) << 24 + ((+76*r/128) & $FF) << 16 + ((-35*r/128) & $FF) << 8 + ((-41*r/128) & $FF)
vidq_		long	              128 << 24 + ((+27*r/128) & $FF) << 16 + ((-67*r/128) & $FF) << 8 + ((+40*r/128) & $FF)
pinmask		long	%1			'pin mask for P0
pindacs		long	%1111_101100000		'pin configuration for video DAC
		'	color base	mode		clocks/pixel	clocks/waitvid
mode		long	$00 << 24 +	$F << 20 +	9 << 13 +	64*9
mode256		long	$00 << 24 +	$F << 20 +	9 << 13 +	256*9
v_bo		long	(lntsc-sntsc-256*9)/2	'left/right border
v_bs		long	100			'before sync		horizontal sync
v_sn		long	269			'sync
v_bc		long	50			'before colorburst
v_cb		long	144			'colorburst
v_bv		long	sntsc-144-50-269-100	'before visible
v_hs		long	135			'high sync		high vertical sync
v_hr		long	lntsc/2-135-100		'high sync remainder
v_ls		long	1564			'low sync		low vertical sync
v_lr		long	lntsc/2-1564-100	'low sync remainder
v_bl		long	lntsc-sntsc		'blank visible
c_sh		long	$1_000000		'sync high
c_sl		long	$2_000000		'sync low
c_cb		long	$1_687800		'colorburst reference color
x		long	0
y		long	0
tmp		long	$0
tmp2		long	$0
tmp3		long	$0

palptr		long	@palette2+$e80'$7c00
hubbuf		long	$8000

one		long	1
VSyncPtr	long	$7bfc

fire_ptr	long	@FireAsm+$e80
fire_par	long	$8000

palette		'

'------------------

        org   0
FireAsm
	rdlong	ffx,fVSyncPtr wz
  if_z	jmp	#$-1
	wrlong	fzero,fVSyncPtr

:gofire
	mov	ffx,#255
:xlp
	mov	ffr,#0
	add	ffr,ffx
	add	ffr,fFrameBuf
	mov	ffw,ffr
	add	ffr,#256-1
	mov	ffy,#192
:ylp
        rdbytec	ffp,ffr
        mov	fft,ffr
        add	fft,#1
        rdbytec	ffn,fft
        add	fft,#1
        add	ffp,ffn        
        rdbytec	ffn,fft
        add	fft,#256-1
        add	ffp,ffn        
        rdbytec	ffn,fft
        add	ffp,ffn
        shr	ffp,#2 wz
  if_nz sub	ffp,#1
        add	ffr,#256      
        wrbyte	ffp,ffw
        add	ffw,#256
        djnz	ffy,#:ylp

        djnz	ffx,#:xlp

	mov	ffx,#160
:xlp1	mov	ffw,#191
	shl	ffw,#8
	add	ffw,fFrameBuf
	mov	fft,seedas
        and	fft,#255
	add	ffw,fft
        mov	fft,#255
        wrbyte	fft,ffw
        ror	seedas,#4
        xor	seedas,seedxo
        add	seedas,seedad
        ror	seedas,#1
        xor	seedas,seedxo
        add	seedas,seedad
        ror	seedas,#2
        xor	seedas,seedxo
        ror	seedas,#1
        add	seedas,seedad
	djnz	ffx,#:xlp1

	mov	ffx,#160
:xlp2	mov	ffw,#190
	shl	ffw,#8
	add	ffw,fFrameBuf
	mov	fft,seedas
        and	fft,#255
	add	ffw,fft
        mov	fft,#255
        wrbyte	fft,ffw
        ror	seedas,#4
        xor	seedas,seedxo
        add	seedas,seedad
        ror	seedas,#1
        xor	seedas,seedxo
        add	seedas,seedad
        ror	seedas,#2
        xor	seedas,seedxo
        ror	seedas,#1
        add	seedas,seedad
	djnz	ffx,#:xlp2

	mov	ffx,#32
:xlp3	qsincos	ang,scaler
	add	ang,angadf
	getqx	ffr
	getqy	ffw
	add	ffr,#128
	add	ffw,#96
	shl	ffw,#8
	add	ffw,ffr
	add	ffw,fFrameBuf
	mov	ffr,#255
	wrbyte	ffr,ffw
	sub	ffw,#256
	wrbyte	ffr,ffw
	add	ffw,#255
	wrbyte	ffr,ffw
	add	ffw,#2
	wrbyte	ffr,ffw
	add	ffw,#255
	wrbyte	ffr,ffw

	djnz	ffx,#:xlp3
	add	ang,angad
        jmp	#FireAsm

ang	long	0
angad	long	1<<24
angadf	long	8<<24
scaler	long	45

ffcntdel	long	60_000_000/30        
                
ffzero		long	0
ffscrsiz	long	256*96/4

seedas		long	$deadbeef
seedxo		long	$beefdead                
seedad		long	$87654321

fFrameBuf	long	$8000
fVSyncPtr	long	$7bfc
fzero		long	0

ffcnt		res	1               
ffp		res	1
ffn		res	1
fft		res	1
ffx		res	1
ffy		res	1
ffr		res	1
ffw		res	1

palette2	file	"fire.bin.pal.bin"
